/**
  *  \file client/actions/gafcode.cc
  *  \brief Global Actions: Friendly Codes
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/actions/gafcode.h"
#include "client/dlg-fcode.h"
#include "cpluslib/format.h"
#include "game/planet.h"
#include "game/ship.h"
#include "ui/commands.h"
#include "ui/msgbox.h"
#include "ui/window.h"
#include "util/nls.h"

/************************ WGlobalRandomFCodeAction ***********************/

class WGlobalRandomFCodeAction::Executer : public WGlobalActionExecuter {
 public:
    bool execute(GObject& obj, bool /*lock*/);
    void finish(int obj, int success);
};

WGlobalActionExecuter*
WGlobalRandomFCodeAction::prepare()
{
    if (UIMessageBox(_("Randomize friendly codes?"), _("Global Actions")).doYesNoDialog()) {
        return new Executer();
    } else {
        return 0;
    }
}

bool
WGlobalRandomFCodeAction::Executer::execute(GObject& obj, bool /*lock*/)
{
    if (!obj.isPlayable(GObject::Playable)) {
        return false;
    } else if (GShip* sh = dynamic_cast<GShip*>(&obj)) {
        sh->setFCode(GFCode::generateRandomFCode());
        return true;
    } else if (GPlanet* pl = dynamic_cast<GPlanet*>(&obj)) {
        pl->setFCode(GFCode::generateRandomFCode());
        return true;
    } else {
        return false;
    }
}

void
WGlobalRandomFCodeAction::Executer::finish(int /*obj*/, int success)
{
    messageBox(format(_("%d friendly code%!1{s%} changed."), success), _("Global Actions"));
}

/************************* WGlobalSetFCodeAction *************************/

class WGlobalSetFCodeAction::Executer : public WGlobalActionExecuter {
    string_t fc;
 public:
    Executer(string_t fc)
        : fc(fc)
        { }
    bool execute(GObject& obj, bool /*lock*/);
    void finish(int obj, int success);
};

bool
WGlobalSetFCodeAction::Executer::execute(GObject& obj, bool /*lock*/)
{
    if (!obj.isPlayable(GObject::Playable)) {
        return false;
    } else if (GShip* sh = dynamic_cast<GShip*>(&obj)) {
        sh->setFCode(fc);
        return true;
    } else if (GPlanet* pl = dynamic_cast<GPlanet*>(&obj)) {
        pl->setFCode(fc);
        return true;
    } else {
        return false;
    }
}

void
WGlobalSetFCodeAction::Executer::finish(int /*obj*/, int success)
{
    messageBox(format(_("%d friendly code%!1{s%} changed."), success), _("Global Actions"));
}

WGlobalActionExecuter*
WGlobalSetFCodeAction::prepare()
{
    WFCodeWindow dialog(_("Set friendly codes"), GFCodeList::getInstance());
    ui_root->centerWidget(dialog);
    ui_root->add(dialog);
    if (dialog.run() == cm_Enter) {
        return new Executer(dialog.getFCode());
    } else {
        return 0;
    }
}
