/**
  *  \file util/strlist.h
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UTIL_STRLIST_H
#define PCC_V2_UTIL_STRLIST_H

#include <list>
#include <utility>
#include "util/strtab.h"

class StringList : public StringTable {
    typedef std::list<std::pair<int,string_t> > list_t;
    list_t data;

 public:
    typedef list_t::const_iterator iterator;

    // Construction
    StringList();
    ~StringList();

    // Insertion
    void       add(int key, const string_t& s);
    void       add(int key, const char* s);
    void       addAtFront(int key, const string_t& s);

    // Random access
    iterator   getItem(int index) const;
    iterator   getItemByKey(int key) const;
    int        getItemIndexByKey(int key) const;

    // Container-style access
    iterator   begin() const;
    iterator   end() const;
    int        getCount() const;

    void       sortAlphabetically();

    // StringTable-style access
    string_t   get(int index) const;
    int        getLastIndex() const;
};

inline StringList::iterator
StringList::begin() const
{
    return data.begin();
}

inline StringList::iterator
StringList::end() const
{
    return data.end();
}

inline int
StringList::getCount() const
{
    return data.size();
}

#endif
